"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .messageresponsedto import MessageResponseDto, MessageResponseDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MessagesResponseDtoTypedDict(TypedDict):
    has_more: bool
    r"""Indicates if there are more messages available"""
    data: List[MessageResponseDtoTypedDict]
    r"""List of messages"""
    page_size: float
    r"""Number of messages per page"""
    page: float
    r"""Current page number"""
    total_count: NotRequired[float]
    r"""Total number of messages available"""


class MessagesResponseDto(BaseModel):
    has_more: Annotated[bool, pydantic.Field(alias="hasMore")]
    r"""Indicates if there are more messages available"""

    data: List[MessageResponseDto]
    r"""List of messages"""

    page_size: Annotated[float, pydantic.Field(alias="pageSize")]
    r"""Number of messages per page"""

    page: float
    r"""Current page number"""

    total_count: Annotated[Optional[float], pydantic.Field(alias="totalCount")] = None
    r"""Total number of messages available"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .patchpreferencechannelsdto import (
    PatchPreferenceChannelsDto,
    PatchPreferenceChannelsDtoTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class BulkUpdateSubscriberPreferenceItemDtoTypedDict(TypedDict):
    channels: PatchPreferenceChannelsDtoTypedDict
    r"""Channel-specific preference settings"""
    workflow_id: str
    r"""Workflow internal _id, identifier or slug"""


class BulkUpdateSubscriberPreferenceItemDto(BaseModel):
    channels: PatchPreferenceChannelsDto
    r"""Channel-specific preference settings"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Workflow internal _id, identifier or slug"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .grouppreferencefilterdetailsdto import (
    GroupPreferenceFilterDetailsDto,
    GroupPreferenceFilterDetailsDtoTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GroupPreferenceFilterDtoTypedDict(TypedDict):
    filter_: GroupPreferenceFilterDetailsDtoTypedDict
    r"""Filter criteria for workflow IDs and tags"""
    enabled: NotRequired[bool]
    r"""Whether the preference is enabled. Used when condition is not provided."""
    condition: NotRequired[Dict[str, Any]]
    r"""Optional condition using JSON Logic rules"""


class GroupPreferenceFilterDto(BaseModel):
    filter_: Annotated[GroupPreferenceFilterDetailsDto, pydantic.Field(alias="filter")]
    r"""Filter criteria for workflow IDs and tags"""

    enabled: Optional[bool] = None
    r"""Whether the preference is enabled. Used when condition is not provided."""

    condition: Optional[Dict[str, Any]] = None
    r"""Optional condition using JSON Logic rules"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BuilderFieldTypeEnum(str, Enum):
    BOOLEAN = "BOOLEAN"
    TEXT = "TEXT"
    DATE = "DATE"
    NUMBER = "NUMBER"
    STATEMENT = "STATEMENT"
    LIST = "LIST"
    MULTI_LIST = "MULTI_LIST"
    GROUP = "GROUP"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ActorTypeEnum(str, Enum):
    r"""The type of the actor, indicating the role in the notification process."""

    NONE = "none"
    USER = "user"
    SYSTEM_ICON = "system_icon"
    SYSTEM_CUSTOM = "system_custom"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .topicsubscriptionresponsedto import (
    TopicSubscriptionResponseDto,
    TopicSubscriptionResponseDtoTypedDict,
)
from novu_py.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, TypedDict


class ListTopicSubscriptionsResponseDtoTypedDict(TypedDict):
    data: List[TopicSubscriptionResponseDtoTypedDict]
    r"""List of returned Topic Subscriptions"""
    next: Nullable[str]
    r"""The cursor for the next page of results, or null if there are no more pages."""
    previous: Nullable[str]
    r"""The cursor for the previous page of results, or null if this is the first page."""
    total_count: float
    r"""The total count of items (up to 50,000)"""
    total_count_capped: bool
    r"""Whether there are more than 50,000 results available"""


class ListTopicSubscriptionsResponseDto(BaseModel):
    data: List[TopicSubscriptionResponseDto]
    r"""List of returned Topic Subscriptions"""

    next: Nullable[str]
    r"""The cursor for the next page of results, or null if there are no more pages."""

    previous: Nullable[str]
    r"""The cursor for the previous page of results, or null if this is the first page."""

    total_count: Annotated[float, pydantic.Field(alias="totalCount")]
    r"""The total count of items (up to 50,000)"""

    total_count_capped: Annotated[bool, pydantic.Field(alias="totalCountCapped")]
    r"""Whether there are more than 50,000 results available"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["next", "previous"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .scheduledto import ScheduleDto, ScheduleDtoTypedDict
from .subscriberpreferencechannels import (
    SubscriberPreferenceChannels,
    SubscriberPreferenceChannelsTypedDict,
)
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SubscriberGlobalPreferenceDtoTypedDict(TypedDict):
    enabled: bool
    r"""Whether notifications are enabled globally"""
    channels: SubscriberPreferenceChannelsTypedDict
    r"""Channel-specific preference settings"""
    schedule: NotRequired[ScheduleDtoTypedDict]
    r"""Subscriber schedule"""


class SubscriberGlobalPreferenceDto(BaseModel):
    enabled: bool
    r"""Whether notifications are enabled globally"""

    channels: SubscriberPreferenceChannels
    r"""Channel-specific preference settings"""

    schedule: Optional[ScheduleDto] = None
    r"""Subscriber schedule"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getcontextresponsedto import GetContextResponseDto, GetContextResponseDtoTypedDict
from .updatecontextrequestdto import (
    UpdateContextRequestDto,
    UpdateContextRequestDtoTypedDict,
)
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ContextsControllerUpdateContextRequestTypedDict(TypedDict):
    id: str
    r"""Context ID"""
    type: str
    r"""Context type"""
    update_context_request_dto: UpdateContextRequestDtoTypedDict
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class ContextsControllerUpdateContextRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Context ID"""

    type: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Context type"""

    update_context_request_dto: Annotated[
        UpdateContextRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class ContextsControllerUpdateContextResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetContextResponseDtoTypedDict


class ContextsControllerUpdateContextResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetContextResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelsettingsdto import ChannelSettingsDto
from dataclasses import dataclass, field
import httpx
from novu_py.models import NovuError
from novu_py.types import BaseModel, OptionalNullable, UNSET
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated


class SubscriberResponseDtoErrorData(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The identifier used to create this subscriber, which typically corresponds to the user ID in your system."""
    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]
    r"""The unique identifier of the organization to which the subscriber belongs."""
    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]
    r"""The unique identifier of the environment associated with this subscriber."""
    deleted: bool
    r"""Indicates whether the subscriber has been deleted."""
    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The timestamp indicating when the subscriber was created, in ISO 8601 format."""
    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""The timestamp indicating when the subscriber was last updated, in ISO 8601 format."""
    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""The internal ID generated by Novu for your subscriber. This ID does not match the `subscriberId` used in your queries. Refer to `subscriberId` for that identifier."""
    first_name: Annotated[OptionalNullable[str], pydantic.Field(alias="firstName")] = (
        UNSET
    )
    r"""The first name of the subscriber."""
    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""The last name of the subscriber."""
    email: OptionalNullable[str] = UNSET
    r"""The email address of the subscriber."""
    phone: OptionalNullable[str] = UNSET
    r"""The phone number of the subscriber."""
    avatar: OptionalNullable[str] = UNSET
    r"""The URL of the subscriber's avatar image."""
    locale: OptionalNullable[str] = UNSET
    r"""The locale setting of the subscriber, indicating their preferred language or region."""
    channels: Optional[List[ChannelSettingsDto]] = None
    r"""An array of channel settings associated with the subscriber."""
    topics: Annotated[
        Optional[List[str]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""An array of topics that the subscriber is subscribed to."""
    is_online: Annotated[OptionalNullable[bool], pydantic.Field(alias="isOnline")] = (
        UNSET
    )
    r"""Indicates whether the subscriber is currently online."""
    last_online_at: Annotated[
        OptionalNullable[str], pydantic.Field(alias="lastOnlineAt")
    ] = UNSET
    r"""The timestamp indicating when the subscriber was last online, in ISO 8601 format."""
    v: Annotated[Optional[float], pydantic.Field(alias="__v")] = None
    r"""The version of the subscriber document."""
    data: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Additional custom data for the subscriber"""
    timezone: OptionalNullable[str] = UNSET
    r"""Timezone of the subscriber"""


@dataclass(unsafe_hash=True)
class SubscriberResponseDtoError(NovuError):
    data: SubscriberResponseDtoErrorData = field(hash=False)

    def __init__(
        self,
        data: SubscriberResponseDtoErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bridgeconfigurationdto import (
    BridgeConfigurationDto,
    BridgeConfigurationDtoTypedDict,
)
from .inboundparsedomaindto import InBoundParseDomainDto, InBoundParseDomainDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateEnvironmentRequestDtoTypedDict(TypedDict):
    name: NotRequired[str]
    identifier: NotRequired[str]
    parent_id: NotRequired[str]
    color: NotRequired[str]
    dns: NotRequired[InBoundParseDomainDtoTypedDict]
    bridge: NotRequired[BridgeConfigurationDtoTypedDict]


class UpdateEnvironmentRequestDto(BaseModel):
    name: Optional[str] = None

    identifier: Optional[str] = None

    parent_id: Annotated[Optional[str], pydantic.Field(alias="parentId")] = None

    color: Optional[str] = None

    dns: Optional[InBoundParseDomainDto] = None

    bridge: Optional[BridgeConfigurationDto] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ChannelTypeEnum(str, Enum):
    r"""Channel type through which the message is sent"""

    IN_APP = "in_app"
    EMAIL = "email"
    SMS = "sms"
    CHAT = "chat"
    PUSH = "push"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .activitiesresponsedto import ActivitiesResponseDto, ActivitiesResponseDtoTypedDict
from .channeltypeenum import ChannelTypeEnum
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NotificationsControllerListNotificationsRequestTypedDict(TypedDict):
    channels: NotRequired[List[ChannelTypeEnum]]
    r"""Array of channel types"""
    templates: NotRequired[List[str]]
    r"""Array of template IDs or a single template ID"""
    emails: NotRequired[List[str]]
    r"""Array of email addresses or a single email address"""
    search: NotRequired[str]
    r"""Search term (deprecated)"""
    subscriber_ids: NotRequired[List[str]]
    r"""Array of subscriber IDs or a single subscriber ID"""
    severity: NotRequired[List[str]]
    r"""Array of severity levels or a single severity level"""
    page: NotRequired[float]
    r"""Page number for pagination"""
    limit: NotRequired[float]
    r"""Limit for pagination"""
    transaction_id: NotRequired[str]
    r"""The transaction ID to filter by"""
    topic_key: NotRequired[str]
    r"""Topic Key for filtering notifications by topic"""
    subscription_id: NotRequired[str]
    r"""Subscription ID for filtering notifications by subscription"""
    context_keys: NotRequired[List[str]]
    r"""Filter by exact context keys, order insensitive (format: \"type:id\")"""
    after: NotRequired[str]
    r"""Date filter for records after this timestamp. Defaults to earliest date allowed by subscription plan"""
    before: NotRequired[str]
    r"""Date filter for records before this timestamp. Defaults to current time of request (now)"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class NotificationsControllerListNotificationsRequest(BaseModel):
    channels: Annotated[
        Optional[List[ChannelTypeEnum]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Array of channel types"""

    templates: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Array of template IDs or a single template ID"""

    emails: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Array of email addresses or a single email address"""

    search: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Search term (deprecated)"""

    subscriber_ids: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="subscriberIds"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Array of subscriber IDs or a single subscriber ID"""

    severity: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Array of severity levels or a single severity level"""

    page: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 0
    r"""Page number for pagination"""

    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Limit for pagination"""

    transaction_id: Annotated[
        Optional[str],
        pydantic.Field(alias="transactionId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The transaction ID to filter by"""

    topic_key: Annotated[
        Optional[str],
        pydantic.Field(alias="topicKey"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Topic Key for filtering notifications by topic"""

    subscription_id: Annotated[
        Optional[str],
        pydantic.Field(alias="subscriptionId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Subscription ID for filtering notifications by subscription"""

    context_keys: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="contextKeys"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by exact context keys, order insensitive (format: \"type:id\")"""

    after: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Date filter for records after this timestamp. Defaults to earliest date allowed by subscription plan"""

    before: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Date filter for records before this timestamp. Defaults to current time of request (now)"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class NotificationsControllerListNotificationsResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ActivitiesResponseDtoTypedDict


class NotificationsControllerListNotificationsResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ActivitiesResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TextAlignEnum(str, Enum):
    r"""Text alignment for the email block"""

    CENTER = "center"
    LEFT = "left"
    RIGHT = "right"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BridgeConfigurationDtoTypedDict(TypedDict):
    url: NotRequired[str]


class BridgeConfigurationDto(BaseModel):
    url: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class MessageActionStatusEnum(str, Enum):
    r"""Status of the message action"""

    PENDING = "pending"
    DONE = "done"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getchannelendpointresponsedto import (
    GetChannelEndpointResponseDto,
    GetChannelEndpointResponseDtoTypedDict,
)
from .updatechannelendpointrequestdto import (
    UpdateChannelEndpointRequestDto,
    UpdateChannelEndpointRequestDtoTypedDict,
)
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChannelEndpointsControllerUpdateChannelEndpointRequestTypedDict(TypedDict):
    identifier: str
    r"""The unique identifier of the channel endpoint"""
    update_channel_endpoint_request_dto: UpdateChannelEndpointRequestDtoTypedDict
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class ChannelEndpointsControllerUpdateChannelEndpointRequest(BaseModel):
    identifier: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the channel endpoint"""

    update_channel_endpoint_request_dto: Annotated[
        UpdateChannelEndpointRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class ChannelEndpointsControllerUpdateChannelEndpointResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetChannelEndpointResponseDtoTypedDict


class ChannelEndpointsControllerUpdateChannelEndpointResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetChannelEndpointResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class MonthlyTypeEnum(str, Enum):
    r"""Type of monthly schedule"""

    EACH = "each"
    ON = "on"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createdsubscriberdto import CreatedSubscriberDto, CreatedSubscriberDtoTypedDict
from .failedoperationdto import FailedOperationDto, FailedOperationDtoTypedDict
from .updatedsubscriberdto import UpdatedSubscriberDto, UpdatedSubscriberDtoTypedDict
from novu_py.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class BulkCreateSubscriberResponseDtoTypedDict(TypedDict):
    updated: List[UpdatedSubscriberDtoTypedDict]
    r"""An array of subscribers that were successfully updated."""
    created: List[CreatedSubscriberDtoTypedDict]
    r"""An array of subscribers that were successfully created."""
    failed: List[FailedOperationDtoTypedDict]
    r"""An array of failed operations with error messages and optional subscriber IDs."""


class BulkCreateSubscriberResponseDto(BaseModel):
    updated: List[UpdatedSubscriberDto]
    r"""An array of subscribers that were successfully updated."""

    created: List[CreatedSubscriberDto]
    r"""An array of subscribers that were successfully created."""

    failed: List[FailedOperationDto]
    r"""An array of failed operations with error messages and optional subscriber IDs."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChannelCredentialsTypedDict(TypedDict):
    webhook_url: NotRequired[str]
    r"""Webhook URL used by chat app integrations. The webhook should be obtained from the chat app provider."""
    channel: NotRequired[str]
    r"""Channel specification for Mattermost chat notifications."""
    device_tokens: NotRequired[List[str]]
    r"""Contains an array of the subscriber device tokens for a given provider. Used on Push integrations."""
    alert_uid: NotRequired[str]
    r"""Alert UID for Grafana on-call webhook payload."""
    title: NotRequired[str]
    r"""Title to be used with Grafana on-call webhook."""
    image_url: NotRequired[str]
    r"""Image URL property for Grafana on-call webhook."""
    state: NotRequired[str]
    r"""State property for Grafana on-call webhook."""
    external_url: NotRequired[str]
    r"""Link to upstream details property for Grafana on-call webhook."""


class ChannelCredentials(BaseModel):
    webhook_url: Annotated[Optional[str], pydantic.Field(alias="webhookUrl")] = None
    r"""Webhook URL used by chat app integrations. The webhook should be obtained from the chat app provider."""

    channel: Optional[str] = None
    r"""Channel specification for Mattermost chat notifications."""

    device_tokens: Annotated[
        Optional[List[str]], pydantic.Field(alias="deviceTokens")
    ] = None
    r"""Contains an array of the subscriber device tokens for a given provider. Used on Push integrations."""

    alert_uid: Annotated[Optional[str], pydantic.Field(alias="alertUid")] = None
    r"""Alert UID for Grafana on-call webhook payload."""

    title: Optional[str] = None
    r"""Title to be used with Grafana on-call webhook."""

    image_url: Annotated[Optional[str], pydantic.Field(alias="imageUrl")] = None
    r"""Image URL property for Grafana on-call webhook."""

    state: Optional[str] = None
    r"""State property for Grafana on-call webhook."""

    external_url: Annotated[Optional[str], pydantic.Field(alias="externalUrl")] = None
    r"""Link to upstream details property for Grafana on-call webhook."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channeltypeenum import ChannelTypeEnum
from .preferenceoverridesourceenum import PreferenceOverrideSourceEnum
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class SubscriberPreferenceOverrideDtoTypedDict(TypedDict):
    channel: ChannelTypeEnum
    r"""Channel type through which the message is sent"""
    source: PreferenceOverrideSourceEnum
    r"""The source of overrides"""


class SubscriberPreferenceOverrideDto(BaseModel):
    channel: ChannelTypeEnum
    r"""Channel type through which the message is sent"""

    source: PreferenceOverrideSourceEnum
    r"""The source of overrides"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createchannelconnectionrequestdto import (
    CreateChannelConnectionRequestDto,
    CreateChannelConnectionRequestDtoTypedDict,
)
from .getchannelconnectionresponsedto import (
    GetChannelConnectionResponseDto,
    GetChannelConnectionResponseDtoTypedDict,
)
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChannelConnectionsControllerCreateChannelConnectionRequestTypedDict(TypedDict):
    create_channel_connection_request_dto: CreateChannelConnectionRequestDtoTypedDict
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class ChannelConnectionsControllerCreateChannelConnectionRequest(BaseModel):
    create_channel_connection_request_dto: Annotated[
        CreateChannelConnectionRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class ChannelConnectionsControllerCreateChannelConnectionResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetChannelConnectionResponseDtoTypedDict


class ChannelConnectionsControllerCreateChannelConnectionResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetChannelConnectionResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DeleteTopicSubscriberIdentifierDtoTypedDict(TypedDict):
    identifier: NotRequired[str]
    r"""Unique identifier for this subscription. If provided, deletes only this specific subscription."""
    subscriber_id: NotRequired[str]
    r"""The subscriber ID. If provided without identifier, deletes all subscriptions for this subscriber within the topic."""


class DeleteTopicSubscriberIdentifierDto(BaseModel):
    identifier: Optional[str] = None
    r"""Unique identifier for this subscription. If provided, deletes only this specific subscription."""

    subscriber_id: Annotated[Optional[str], pydantic.Field(alias="subscriberId")] = None
    r"""The subscriber ID. If provided without identifier, deletes all subscriptions for this subscriber within the topic."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class LookBackWindowDtoUnit(str, Enum):
    r"""Unit of time for the look-back window."""

    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"


class LookBackWindowDtoTypedDict(TypedDict):
    amount: float
    r"""Amount of time for the look-back window."""
    unit: LookBackWindowDtoUnit
    r"""Unit of time for the look-back window."""


class LookBackWindowDto(BaseModel):
    amount: float
    r"""Amount of time for the look-back window."""

    unit: LookBackWindowDtoUnit
    r"""Unit of time for the look-back window."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class SubscriptionsDeleteErrorDtoTypedDict(TypedDict):
    subscriber_id: str
    r"""The subscriber ID that failed"""
    code: str
    r"""The error code"""
    message: str
    r"""The error message"""


class SubscriptionsDeleteErrorDto(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The subscriber ID that failed"""

    code: str
    r"""The error code"""

    message: str
    r"""The error message"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatstepupsertdto import ChatStepUpsertDto, ChatStepUpsertDtoTypedDict
from .customstepupsertdto import CustomStepUpsertDto, CustomStepUpsertDtoTypedDict
from .delaystepupsertdto import DelayStepUpsertDto, DelayStepUpsertDtoTypedDict
from .digeststepupsertdto import DigestStepUpsertDto, DigestStepUpsertDtoTypedDict
from .emailstepupsertdto import EmailStepUpsertDto, EmailStepUpsertDtoTypedDict
from .inappstepupsertdto import InAppStepUpsertDto, InAppStepUpsertDtoTypedDict
from .preferencesrequestdto import PreferencesRequestDto, PreferencesRequestDtoTypedDict
from .pushstepupsertdto import PushStepUpsertDto, PushStepUpsertDtoTypedDict
from .severitylevelenum import SeverityLevelEnum
from .smsstepupsertdto import SmsStepUpsertDto, SmsStepUpsertDtoTypedDict
from .throttlestepupsertdto import ThrottleStepUpsertDto, ThrottleStepUpsertDtoTypedDict
from .workflowcreationsourceenum import WorkflowCreationSourceEnum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from novu_py.utils import get_discriminator
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


StepsModelTypedDict = TypeAliasType(
    "StepsModelTypedDict",
    Union[
        InAppStepUpsertDtoTypedDict,
        EmailStepUpsertDtoTypedDict,
        SmsStepUpsertDtoTypedDict,
        PushStepUpsertDtoTypedDict,
        ChatStepUpsertDtoTypedDict,
        DelayStepUpsertDtoTypedDict,
        DigestStepUpsertDtoTypedDict,
        ThrottleStepUpsertDtoTypedDict,
        CustomStepUpsertDtoTypedDict,
    ],
)


StepsModel = Annotated[
    Union[
        Annotated[InAppStepUpsertDto, Tag("in_app")],
        Annotated[EmailStepUpsertDto, Tag("email")],
        Annotated[SmsStepUpsertDto, Tag("sms")],
        Annotated[PushStepUpsertDto, Tag("push")],
        Annotated[ChatStepUpsertDto, Tag("chat")],
        Annotated[DelayStepUpsertDto, Tag("delay")],
        Annotated[DigestStepUpsertDto, Tag("digest")],
        Annotated[ThrottleStepUpsertDto, Tag("throttle")],
        Annotated[CustomStepUpsertDto, Tag("custom")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class CreateWorkflowDtoTypedDict(TypedDict):
    name: str
    r"""Name of the workflow"""
    workflow_id: str
    r"""Unique identifier for the workflow"""
    steps: List[StepsModelTypedDict]
    r"""Steps of the workflow"""
    description: NotRequired[str]
    r"""Description of the workflow"""
    tags: NotRequired[List[str]]
    r"""Tags associated with the workflow"""
    active: NotRequired[bool]
    r"""Whether the workflow is active"""
    validate_payload: NotRequired[bool]
    r"""Enable or disable payload schema validation"""
    payload_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""The payload JSON Schema for the workflow"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this workflow"""
    source: NotRequired[WorkflowCreationSourceEnum]
    r"""Source of workflow creation"""
    preferences: NotRequired[PreferencesRequestDtoTypedDict]
    r"""Workflow preferences"""
    severity: NotRequired[SeverityLevelEnum]
    r"""Severity of the workflow"""


class CreateWorkflowDto(BaseModel):
    name: str
    r"""Name of the workflow"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Unique identifier for the workflow"""

    steps: List[StepsModel]
    r"""Steps of the workflow"""

    description: Optional[str] = None
    r"""Description of the workflow"""

    tags: Optional[List[str]] = None
    r"""Tags associated with the workflow"""

    active: Optional[bool] = False
    r"""Whether the workflow is active"""

    validate_payload: Annotated[
        Optional[bool], pydantic.Field(alias="validatePayload")
    ] = None
    r"""Enable or disable payload schema validation"""

    payload_schema: Annotated[
        OptionalNullable[Dict[str, Any]], pydantic.Field(alias="payloadSchema")
    ] = UNSET
    r"""The payload JSON Schema for the workflow"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this workflow"""

    source: Annotated[
        Optional[WorkflowCreationSourceEnum], pydantic.Field(alias="__source")
    ] = WorkflowCreationSourceEnum.EDITOR
    r"""Source of workflow creation"""

    preferences: Optional[PreferencesRequestDto] = None
    r"""Workflow preferences"""

    severity: Optional[SeverityLevelEnum] = None
    r"""Severity of the workflow"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "tags",
            "active",
            "validatePayload",
            "payloadSchema",
            "isTranslationEnabled",
            "__source",
            "preferences",
            "severity",
        ]
        nullable_fields = ["payloadSchema"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .messageactionresult import MessageActionResult, MessageActionResultTypedDict
from .messageactionstatusenum import MessageActionStatusEnum
from .messagebutton import MessageButton, MessageButtonTypedDict
from novu_py.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class MessageActionTypedDict(TypedDict):
    status: NotRequired[MessageActionStatusEnum]
    r"""Status of the message action"""
    buttons: NotRequired[List[MessageButtonTypedDict]]
    r"""List of buttons associated with the message action"""
    result: NotRequired[MessageActionResultTypedDict]
    r"""Result of the message action"""


class MessageAction(BaseModel):
    status: Optional[MessageActionStatusEnum] = None
    r"""Status of the message action"""

    buttons: Optional[List[MessageButton]] = None
    r"""List of buttons associated with the message action"""

    result: Optional[MessageActionResult] = None
    r"""Result of the message action"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


FeedIdentifierTypedDict = TypeAliasType(
    "FeedIdentifierTypedDict", Union[str, List[str]]
)
r"""Optional feed identifier or array of feed identifiers"""


FeedIdentifier = TypeAliasType("FeedIdentifier", Union[str, List[str]])
r"""Optional feed identifier or array of feed identifiers"""


class MarkAs(str, Enum):
    r"""Mark all subscriber messages as read, unread, seen or unseen"""

    READ = "read"
    SEEN = "seen"
    UNREAD = "unread"
    UNSEEN = "unseen"


class MarkAllMessageAsRequestDtoTypedDict(TypedDict):
    mark_as: MarkAs
    r"""Mark all subscriber messages as read, unread, seen or unseen"""
    feed_identifier: NotRequired[FeedIdentifierTypedDict]
    r"""Optional feed identifier or array of feed identifiers"""


class MarkAllMessageAsRequestDto(BaseModel):
    mark_as: Annotated[MarkAs, pydantic.Field(alias="markAs")]
    r"""Mark all subscriber messages as read, unread, seen or unseen"""

    feed_identifier: Annotated[
        Optional[FeedIdentifier], pydantic.Field(alias="feedIdentifier")
    ] = None
    r"""Optional feed identifier or array of feed identifiers"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


MessageIDTypedDict = TypeAliasType("MessageIDTypedDict", Union[str, List[str]])


MessageID = TypeAliasType("MessageID", Union[str, List[str]])


class MessageMarkAsRequestDtoMarkAs(str, Enum):
    READ = "read"
    SEEN = "seen"
    UNREAD = "unread"
    UNSEEN = "unseen"


class MessageMarkAsRequestDtoTypedDict(TypedDict):
    message_id: MessageIDTypedDict
    mark_as: MessageMarkAsRequestDtoMarkAs


class MessageMarkAsRequestDto(BaseModel):
    message_id: Annotated[MessageID, pydantic.Field(alias="messageId")]

    mark_as: Annotated[MessageMarkAsRequestDtoMarkAs, pydantic.Field(alias="markAs")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GroupPreferenceFilterDetailsDtoTypedDict(TypedDict):
    workflow_ids: NotRequired[List[str]]
    r"""List of workflow identifiers"""
    tags: NotRequired[List[str]]
    r"""List of tags"""


class GroupPreferenceFilterDetailsDto(BaseModel):
    workflow_ids: Annotated[
        Optional[List[str]], pydantic.Field(alias="workflowIds")
    ] = None
    r"""List of workflow identifiers"""

    tags: Optional[List[str]] = None
    r"""List of tags"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WorkflowCreationSourceEnum(str, Enum):
    r"""Source of workflow creation"""

    TEMPLATE_STORE = "template_store"
    EDITOR = "editor"
    NOTIFICATION_DIRECTORY = "notification_directory"
    ONBOARDING_DIGEST_DEMO = "onboarding_digest_demo"
    ONBOARDING_IN_APP = "onboarding_in_app"
    EMPTY_STATE = "empty_state"
    DROPDOWN = "dropdown"
    ONBOARDING_GET_STARTED = "onboarding_get_started"
    BRIDGE = "bridge"
    DASHBOARD = "dashboard"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class APIKeyDtoTypedDict(TypedDict):
    key: str
    r"""API key"""
    user_id: str
    r"""User ID associated with the API key"""
    hash: NotRequired[str]
    r"""Hashed representation of the API key"""


class APIKeyDto(BaseModel):
    key: str
    r"""API key"""

    user_id: Annotated[str, pydantic.Field(alias="_userId")]
    r"""User ID associated with the API key"""

    hash: Optional[str] = None
    r"""Hashed representation of the API key"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SubscriberPreferenceChannelsTypedDict(TypedDict):
    email: NotRequired[bool]
    r"""Email channel preference"""
    sms: NotRequired[bool]
    r"""SMS channel preference"""
    in_app: NotRequired[bool]
    r"""In-app channel preference"""
    chat: NotRequired[bool]
    r"""Chat channel preference"""
    push: NotRequired[bool]
    r"""Push notification channel preference"""


class SubscriberPreferenceChannels(BaseModel):
    email: Optional[bool] = None
    r"""Email channel preference"""

    sms: Optional[bool] = None
    r"""SMS channel preference"""

    in_app: Optional[bool] = None
    r"""In-app channel preference"""

    chat: Optional[bool] = None
    r"""Chat channel preference"""

    push: Optional[bool] = None
    r"""Push notification channel preference"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listchannelendpointsresponsedto import (
    ListChannelEndpointsResponseDto,
    ListChannelEndpointsResponseDtoTypedDict,
)
from .providersidenum import ProvidersIDEnum
from enum import Enum
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChannelEndpointsControllerListChannelEndpointsQueryParamOrderDirection(str, Enum):
    r"""Direction of sorting"""

    ASC = "ASC"
    DESC = "DESC"


class ChannelEndpointsControllerListChannelEndpointsQueryParamChannel(str, Enum):
    r"""Channel type to filter results."""

    IN_APP = "in_app"
    EMAIL = "email"
    SMS = "sms"
    CHAT = "chat"
    PUSH = "push"


class ChannelEndpointsControllerListChannelEndpointsRequestTypedDict(TypedDict):
    after: NotRequired[str]
    r"""Cursor for pagination indicating the starting point after which to fetch results."""
    before: NotRequired[str]
    r"""Cursor for pagination indicating the ending point before which to fetch results."""
    limit: NotRequired[float]
    r"""Limit the number of items to return (max 100)"""
    order_direction: NotRequired[
        ChannelEndpointsControllerListChannelEndpointsQueryParamOrderDirection
    ]
    r"""Direction of sorting"""
    order_by: NotRequired[str]
    r"""Field to order by"""
    include_cursor: NotRequired[bool]
    r"""Include cursor item in response"""
    subscriber_id: NotRequired[str]
    r"""The subscriber ID to filter results by"""
    context_keys: NotRequired[List[str]]
    r"""Filter by exact context keys, order insensitive (format: \"type:id\")"""
    channel: NotRequired[
        ChannelEndpointsControllerListChannelEndpointsQueryParamChannel
    ]
    r"""Channel type to filter results."""
    provider_id: NotRequired[ProvidersIDEnum]
    r"""Filter by provider identifier (e.g., sendgrid, twilio, slack, etc.)."""
    integration_identifier: NotRequired[str]
    r"""Integration identifier to filter results."""
    connection_identifier: NotRequired[str]
    r"""Connection identifier to filter results."""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class ChannelEndpointsControllerListChannelEndpointsRequest(BaseModel):
    after: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Cursor for pagination indicating the starting point after which to fetch results."""

    before: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Cursor for pagination indicating the ending point before which to fetch results."""

    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Limit the number of items to return (max 100)"""

    order_direction: Annotated[
        Optional[
            ChannelEndpointsControllerListChannelEndpointsQueryParamOrderDirection
        ],
        pydantic.Field(alias="orderDirection"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Direction of sorting"""

    order_by: Annotated[
        Optional[str],
        pydantic.Field(alias="orderBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Field to order by"""

    include_cursor: Annotated[
        Optional[bool],
        pydantic.Field(alias="includeCursor"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Include cursor item in response"""

    subscriber_id: Annotated[
        Optional[str],
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The subscriber ID to filter results by"""

    context_keys: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="contextKeys"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by exact context keys, order insensitive (format: \"type:id\")"""

    channel: Annotated[
        Optional[ChannelEndpointsControllerListChannelEndpointsQueryParamChannel],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Channel type to filter results."""

    provider_id: Annotated[
        Optional[ProvidersIDEnum],
        pydantic.Field(alias="providerId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by provider identifier (e.g., sendgrid, twilio, slack, etc.)."""

    integration_identifier: Annotated[
        Optional[str],
        pydantic.Field(alias="integrationIdentifier"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Integration identifier to filter results."""

    connection_identifier: Annotated[
        Optional[str],
        pydantic.Field(alias="connectionIdentifier"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Connection identifier to filter results."""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class ChannelEndpointsControllerListChannelEndpointsResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ListChannelEndpointsResponseDtoTypedDict


class ChannelEndpointsControllerListChannelEndpointsResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ListChannelEndpointsResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict
from typing_extensions import Annotated, TypedDict


class ResourceType(str, Enum):
    r"""The resource type to associate translation with"""

    WORKFLOW = "workflow"
    LAYOUT = "layout"


class CreateTranslationRequestDtoTypedDict(TypedDict):
    resource_id: str
    r"""The resource ID to associate translation with. Accepts identifier or slug format"""
    resource_type: ResourceType
    r"""The resource type to associate translation with"""
    locale: str
    r"""Locale code (e.g., en_US, es_ES)"""
    content: Dict[str, Any]
    r"""Translation content as JSON object"""


class CreateTranslationRequestDto(BaseModel):
    resource_id: Annotated[str, pydantic.Field(alias="resourceId")]
    r"""The resource ID to associate translation with. Accepts identifier or slug format"""

    resource_type: Annotated[ResourceType, pydantic.Field(alias="resourceType")]
    r"""The resource type to associate translation with"""

    locale: str
    r"""Locale code (e.g., en_US, es_ES)"""

    content: Dict[str, Any]
    r"""Translation content as JSON object"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .layoutresponsedto import LayoutResponseDto, LayoutResponseDtoTypedDict
from .updatelayoutdto import UpdateLayoutDto, UpdateLayoutDtoTypedDict
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LayoutsControllerUpdateRequestTypedDict(TypedDict):
    layout_id: str
    update_layout_dto: UpdateLayoutDtoTypedDict
    r"""Layout update details"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class LayoutsControllerUpdateRequest(BaseModel):
    layout_id: Annotated[
        str,
        pydantic.Field(alias="layoutId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    update_layout_dto: Annotated[
        UpdateLayoutDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Layout update details"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class LayoutsControllerUpdateResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: LayoutResponseDtoTypedDict


class LayoutsControllerUpdateResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: LayoutResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .unseencountresponse import UnseenCountResponse, UnseenCountResponseTypedDict
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscribersV1ControllerGetUnseenCountRequestTypedDict(TypedDict):
    subscriber_id: str
    seen: NotRequired[bool]
    r"""Indicates whether to count seen notifications."""
    limit: NotRequired[float]
    r"""The maximum number of notifications to return."""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class SubscribersV1ControllerGetUnseenCountRequest(BaseModel):
    subscriber_id: Annotated[
        str,
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    seen: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""Indicates whether to count seen notifications."""

    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100
    r"""The maximum number of notifications to return."""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class SubscribersV1ControllerGetUnseenCountResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: UnseenCountResponseTypedDict


class SubscribersV1ControllerGetUnseenCountResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: UnseenCountResponse



import json
import random
import time
from typing import Union

import httpx

from .types import (
    BeforeRequestHook,
    AfterSuccessHook,
    BeforeRequestContext,
    AfterSuccessContext
)


def generate_idempotency_key():
    """
    Generate a unique idempotency key.

    :return: A unique idempotency key as a string
    """
    timestamp = int(time.time() * 1000)  # Current time in milliseconds
    random_string = ''.join(random.sample('abcdefghijklmnopqrstuvwxyz0123456789', 9))  # Unique alphanumeric string
    return f"{timestamp}{random_string}"

def clean_headers(headers: dict) -> dict:
    """
    Remove problematic headers from the response.

    :param headers: Original headers
    :return: Cleaned headers
    """
    headers.pop('content-encoding', None)
    headers.pop('transfer-encoding', None)
    return headers

class NovuHooks(BeforeRequestHook, AfterSuccessHook):
    def before_request(self, hook_ctx: BeforeRequestContext, request: httpx.Request) -> Union[httpx.Request, Exception]:
        """
        Modify the request before sending.

        :param hook_ctx: Context for the before request hook
        :param request: The request to be modified
        :return: Modified request
        """
        auth_key = 'authorization'
        idempotency_key = 'idempotency-key'
        api_key_prefix = 'ApiKey'

        # Create a copy of headers
        headers = dict(request.headers or {})

        # Check and modify authorization header
        if auth_key in headers:
            key = headers[auth_key]
            if key and not key.startswith(api_key_prefix):
                headers[auth_key] = f"{api_key_prefix} {key}"

        # Add idempotency key if not present
        if idempotency_key not in headers or not headers[idempotency_key]:
            headers[idempotency_key] = generate_idempotency_key()

        # Recreate the request with modified headers
        return httpx.Request(
            method=request.method,
            url=request.url,
            headers=headers,
            content=request.content,
            extensions=request.extensions
        )

    def after_success(self, hook_ctx: AfterSuccessContext, response: httpx.Response) -> Union[httpx.Response, Exception]:
        """
        Modify the response after a successful request.

        - Removes problematic headers.
        - Extracts the 'data' key if the response contains only that key.

        :param hook_ctx: Context for the after success hook
        :param response: The response to be potentially modified
        :return: Modified or original response
        """
        # Check content type
        content_type = response.headers.get('Content-Type', '')

        # Return early for empty or HTML responses
        if response.text == '' or 'text/html' in content_type:
            return response

        try:
            json_response = response.json()
        except ValueError:  # Handle JSONDecodeError
            return response

        # Check if the response contains a single 'data' key
        if isinstance(json_response, dict) and len(json_response) == 1 and 'data' in json_response:
            # Create a new HTTPX response
            new_response = httpx.Response(
                status_code=response.status_code,
                headers=clean_headers(dict(response.headers)),
                content=json.dumps(json_response['data']).encode('utf-8'),
                request=response.request
            )
            return new_response

        return response



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from novu_py import models, utils
from novu_py._hooks import HookContext
from novu_py.credentials import Credentials
from novu_py.novu_messages import NovuMessages
from novu_py.novu_notifications import NovuNotifications
from novu_py.novu_topics import NovuTopics
from novu_py.preferences import Preferences
from novu_py.properties import Properties
from novu_py.types import BaseModel, OptionalNullable, UNSET
from novu_py.utils import get_security_from_env
from novu_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class Subscribers(BaseSDK):
    r"""A subscriber in Novu represents someone who should receive a message. A subscriber's profile information contains important attributes about the subscriber that will be used in messages (name, email). The subscriber object can contain other key-value pairs that can be used to further personalize your messages.
    https://docs.novu.co/subscribers/subscribers
    """

    preferences: Preferences
    topics: NovuTopics
    credentials: Credentials
    messages: NovuMessages
    notifications: NovuNotifications
    properties: Properties

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.preferences = Preferences(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.topics = NovuTopics(self.sdk_configuration, parent_ref=self.parent_ref)
        self.credentials = Credentials(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.messages = NovuMessages(self.sdk_configuration, parent_ref=self.parent_ref)
        self.notifications = NovuNotifications(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.properties = Properties(self.sdk_configuration, parent_ref=self.parent_ref)

    def search(
        self,
        *,
        request: Union[
            models.SubscribersControllerSearchSubscribersRequest,
            models.SubscribersControllerSearchSubscribersRequestTypedDict,
        ] = models.SubscribersControllerSearchSubscribersRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerSearchSubscribersResponse:
        r"""Search subscribers

        Search subscribers by their **email**, **phone**, **subscriberId** and **name**.
        The search is case sensitive and supports pagination.Checkout all available filters in the query section.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, models.SubscribersControllerSearchSubscribersRequest
            )
        request = cast(models.SubscribersControllerSearchSubscribersRequest, request)

        req = self._build_request(
            method="GET",
            path="/v2/subscribers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_searchSubscribers",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerSearchSubscribersResponse(
                result=unmarshal_json_response(
                    models.ListSubscribersResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def search_async(
        self,
        *,
        request: Union[
            models.SubscribersControllerSearchSubscribersRequest,
            models.SubscribersControllerSearchSubscribersRequestTypedDict,
        ] = models.SubscribersControllerSearchSubscribersRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerSearchSubscribersResponse:
        r"""Search subscribers

        Search subscribers by their **email**, **phone**, **subscriberId** and **name**.
        The search is case sensitive and supports pagination.Checkout all available filters in the query section.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, models.SubscribersControllerSearchSubscribersRequest
            )
        request = cast(models.SubscribersControllerSearchSubscribersRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/v2/subscribers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_searchSubscribers",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerSearchSubscribersResponse(
                result=unmarshal_json_response(
                    models.ListSubscribersResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        create_subscriber_request_dto: Union[
            models.CreateSubscriberRequestDto,
            models.CreateSubscriberRequestDtoTypedDict,
        ],
        fail_if_exists: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerCreateSubscriberResponse:
        r"""Create a subscriber

        Create a subscriber with the subscriber attributes.
        **subscriberId** is a required field, rest other fields are optional, if the subscriber already exists, it will be updated

        :param create_subscriber_request_dto:
        :param fail_if_exists: If true, the request will fail if a subscriber with the same subscriberId already exists
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerCreateSubscriberRequest(
            fail_if_exists=fail_if_exists,
            idempotency_key=idempotency_key,
            create_subscriber_request_dto=utils.get_pydantic_model(
                create_subscriber_request_dto, models.CreateSubscriberRequestDto
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v2/subscribers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_subscriber_request_dto,
                False,
                False,
                "json",
                models.CreateSubscriberRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_createSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.SubscribersControllerCreateSubscriberResponse(
                result=unmarshal_json_response(models.SubscriberResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(
                models.SubscriberResponseDtoErrorData, http_res
            )
            raise models.SubscriberResponseDtoError(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        create_subscriber_request_dto: Union[
            models.CreateSubscriberRequestDto,
            models.CreateSubscriberRequestDtoTypedDict,
        ],
        fail_if_exists: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerCreateSubscriberResponse:
        r"""Create a subscriber

        Create a subscriber with the subscriber attributes.
        **subscriberId** is a required field, rest other fields are optional, if the subscriber already exists, it will be updated

        :param create_subscriber_request_dto:
        :param fail_if_exists: If true, the request will fail if a subscriber with the same subscriberId already exists
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerCreateSubscriberRequest(
            fail_if_exists=fail_if_exists,
            idempotency_key=idempotency_key,
            create_subscriber_request_dto=utils.get_pydantic_model(
                create_subscriber_request_dto, models.CreateSubscriberRequestDto
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v2/subscribers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_subscriber_request_dto,
                False,
                False,
                "json",
                models.CreateSubscriberRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_createSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.SubscribersControllerCreateSubscriberResponse(
                result=unmarshal_json_response(models.SubscriberResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(
                models.SubscriberResponseDtoErrorData, http_res
            )
            raise models.SubscriberResponseDtoError(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def retrieve(
        self,
        *,
        subscriber_id: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerGetSubscriberResponse:
        r"""Retrieve a subscriber

        Retrieve a subscriber by its unique key identifier **subscriberId**.
        **subscriberId** field is required.

        :param subscriber_id:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerGetSubscriberRequest(
            subscriber_id=subscriber_id,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/v2/subscribers/{subscriberId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_getSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerGetSubscriberResponse(
                result=unmarshal_json_response(models.SubscriberResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def retrieve_async(
        self,
        *,
        subscriber_id: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerGetSubscriberResponse:
        r"""Retrieve a subscriber

        Retrieve a subscriber by its unique key identifier **subscriberId**.
        **subscriberId** field is required.

        :param subscriber_id:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerGetSubscriberRequest(
            subscriber_id=subscriber_id,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/v2/subscribers/{subscriberId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_getSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerGetSubscriberResponse(
                result=unmarshal_json_response(models.SubscriberResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def patch(
        self,
        *,
        subscriber_id: str,
        patch_subscriber_request_dto: Union[
            models.PatchSubscriberRequestDto, models.PatchSubscriberRequestDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerPatchSubscriberResponse:
        r"""Update a subscriber

        Update a subscriber by its unique key identifier **subscriberId**.
        **subscriberId** is a required field, rest other fields are optional

        :param subscriber_id:
        :param patch_subscriber_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerPatchSubscriberRequest(
            subscriber_id=subscriber_id,
            idempotency_key=idempotency_key,
            patch_subscriber_request_dto=utils.get_pydantic_model(
                patch_subscriber_request_dto, models.PatchSubscriberRequestDto
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v2/subscribers/{subscriberId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.patch_subscriber_request_dto,
                False,
                False,
                "json",
                models.PatchSubscriberRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_patchSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerPatchSubscriberResponse(
                result=unmarshal_json_response(models.SubscriberResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def patch_async(
        self,
        *,
        subscriber_id: str,
        patch_subscriber_request_dto: Union[
            models.PatchSubscriberRequestDto, models.PatchSubscriberRequestDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerPatchSubscriberResponse:
        r"""Update a subscriber

        Update a subscriber by its unique key identifier **subscriberId**.
        **subscriberId** is a required field, rest other fields are optional

        :param subscriber_id:
        :param patch_subscriber_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerPatchSubscriberRequest(
            subscriber_id=subscriber_id,
            idempotency_key=idempotency_key,
            patch_subscriber_request_dto=utils.get_pydantic_model(
                patch_subscriber_request_dto, models.PatchSubscriberRequestDto
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v2/subscribers/{subscriberId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.patch_subscriber_request_dto,
                False,
                False,
                "json",
                models.PatchSubscriberRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_patchSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerPatchSubscriberResponse(
                result=unmarshal_json_response(models.SubscriberResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        subscriber_id: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerRemoveSubscriberResponse:
        r"""Delete a subscriber

        Deletes a subscriber entity from the Novu platform along with associated messages, preferences, and topic subscriptions.
        **subscriberId** is a required field.

        :param subscriber_id:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerRemoveSubscriberRequest(
            subscriber_id=subscriber_id,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="DELETE",
            path="/v2/subscribers/{subscriberId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_removeSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerRemoveSubscriberResponse(
                result=unmarshal_json_response(
                    models.RemoveSubscriberResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        subscriber_id: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersControllerRemoveSubscriberResponse:
        r"""Delete a subscriber

        Deletes a subscriber entity from the Novu platform along with associated messages, preferences, and topic subscriptions.
        **subscriberId** is a required field.

        :param subscriber_id:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersControllerRemoveSubscriberRequest(
            subscriber_id=subscriber_id,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v2/subscribers/{subscriberId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersController_removeSubscriber",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SubscribersControllerRemoveSubscriberResponse(
                result=unmarshal_json_response(
                    models.RemoveSubscriberResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create_bulk(
        self,
        *,
        bulk_subscriber_create_dto: Union[
            models.BulkSubscriberCreateDto, models.BulkSubscriberCreateDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersV1ControllerBulkCreateSubscribersResponse:
        r"""Bulk create subscribers


        Using this endpoint multiple subscribers can be created at once. The bulk API is limited to 500 subscribers per request.


        :param bulk_subscriber_create_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersV1ControllerBulkCreateSubscribersRequest(
            idempotency_key=idempotency_key,
            bulk_subscriber_create_dto=utils.get_pydantic_model(
                bulk_subscriber_create_dto, models.BulkSubscriberCreateDto
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/subscribers/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.bulk_subscriber_create_dto,
                False,
                False,
                "json",
                models.BulkSubscriberCreateDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersV1Controller_bulkCreateSubscribers",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.SubscribersV1ControllerBulkCreateSubscribersResponse(
                result=unmarshal_json_response(
                    models.BulkCreateSubscriberResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_bulk_async(
        self,
        *,
        bulk_subscriber_create_dto: Union[
            models.BulkSubscriberCreateDto, models.BulkSubscriberCreateDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SubscribersV1ControllerBulkCreateSubscribersResponse:
        r"""Bulk create subscribers


        Using this endpoint multiple subscribers can be created at once. The bulk API is limited to 500 subscribers per request.


        :param bulk_subscriber_create_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SubscribersV1ControllerBulkCreateSubscribersRequest(
            idempotency_key=idempotency_key,
            bulk_subscriber_create_dto=utils.get_pydantic_model(
                bulk_subscriber_create_dto, models.BulkSubscriberCreateDto
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/subscribers/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.bulk_subscriber_create_dto,
                False,
                False,
                "json",
                models.BulkSubscriberCreateDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="SubscribersV1Controller_bulkCreateSubscribers",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.SubscribersV1ControllerBulkCreateSubscribersResponse(
                result=unmarshal_json_response(
                    models.BulkCreateSubscriberResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class Target(str, Enum):
    r"""Target window for the redirection."""

    SELF = "_self"
    BLANK = "_blank"
    PARENT = "_parent"
    TOP = "_top"
    UNFENCED_TOP = "_unfencedTop"


class RedirectDtoTypedDict(TypedDict):
    url: NotRequired[str]
    r"""URL for redirection. Must be a valid URL or start with / or {{ variable }}."""
    target: NotRequired[Target]
    r"""Target window for the redirection."""


class RedirectDto(BaseModel):
    url: Optional[str] = None
    r"""URL for redirection. Must be a valid URL or start with / or {{ variable }}."""

    target: Optional[Target] = Target.SELF
    r"""Target window for the redirection."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelpreferencedto import ChannelPreferenceDto, ChannelPreferenceDtoTypedDict
from .workflowpreferencedto import WorkflowPreferenceDto, WorkflowPreferenceDtoTypedDict
from novu_py.types import BaseModel
from typing import Dict
from typing_extensions import TypedDict


AllTypedDict = WorkflowPreferenceDtoTypedDict
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


All = WorkflowPreferenceDto
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


class WorkflowPreferencesDtoTypedDict(TypedDict):
    all: AllTypedDict
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""
    channels: Dict[str, ChannelPreferenceDtoTypedDict]
    r"""Preferences for different communication channels"""


class WorkflowPreferencesDto(BaseModel):
    all: All
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""

    channels: Dict[str, ChannelPreferenceDto]
    r"""Preferences for different communication channels"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .generatelayoutpreviewresponsedto import (
    GenerateLayoutPreviewResponseDto,
    GenerateLayoutPreviewResponseDtoTypedDict,
)
from .layoutpreviewrequestdto import (
    LayoutPreviewRequestDto,
    LayoutPreviewRequestDtoTypedDict,
)
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LayoutsControllerGeneratePreviewRequestTypedDict(TypedDict):
    layout_id: str
    layout_preview_request_dto: LayoutPreviewRequestDtoTypedDict
    r"""Layout preview generation details"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class LayoutsControllerGeneratePreviewRequest(BaseModel):
    layout_id: Annotated[
        str,
        pydantic.Field(alias="layoutId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    layout_preview_request_dto: Annotated[
        LayoutPreviewRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Layout preview generation details"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class LayoutsControllerGeneratePreviewResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GenerateLayoutPreviewResponseDtoTypedDict


class LayoutsControllerGeneratePreviewResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GenerateLayoutPreviewResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .stepissuesdto import StepIssuesDto, StepIssuesDtoTypedDict
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class StepListResponseDtoTypedDict(TypedDict):
    slug: str
    r"""Slug of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    issues: NotRequired[StepIssuesDtoTypedDict]
    r"""Issues associated with the step"""


class StepListResponseDto(BaseModel):
    slug: str
    r"""Slug of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    issues: Optional[StepIssuesDto] = None
    r"""Issues associated with the step"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class SyncWorkflowDtoTypedDict(TypedDict):
    target_environment_id: str
    r"""Target environment identifier to sync the workflow to"""


class SyncWorkflowDto(BaseModel):
    target_environment_id: Annotated[str, pydantic.Field(alias="targetEnvironmentId")]
    r"""Target environment identifier to sync the workflow to"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PreferenceOverrideSourceEnum(str, Enum):
    r"""The source of overrides"""

    SUBSCRIBER = "subscriber"
    TEMPLATE = "template"
    WORKFLOW_OVERRIDE = "workflowOverride"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class WebhookEndpointDtoTypedDict(TypedDict):
    url: str
    r"""Webhook URL"""
    channel: NotRequired[str]
    r"""Optional channel identifier"""


class WebhookEndpointDto(BaseModel):
    url: str
    r"""Webhook URL"""

    channel: Optional[str] = None
    r"""Optional channel identifier"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ThrottleControlDtoType(str, Enum):
    r"""The type of throttle window."""

    FIXED = "fixed"
    DYNAMIC = "dynamic"


class ThrottleControlDtoUnit(str, Enum):
    r"""The unit of time for the throttle window (required for fixed type)."""

    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"


class ThrottleControlDtoTypedDict(TypedDict):
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    type: NotRequired[ThrottleControlDtoType]
    r"""The type of throttle window."""
    amount: NotRequired[float]
    r"""The amount of time for the throttle window (required for fixed type)."""
    unit: NotRequired[ThrottleControlDtoUnit]
    r"""The unit of time for the throttle window (required for fixed type)."""
    dynamic_key: NotRequired[str]
    r"""Key path to retrieve dynamic window value (required for dynamic type)."""
    threshold: NotRequired[float]
    r"""The maximum number of executions allowed within the window. Defaults to 1."""
    throttle_key: NotRequired[str]
    r"""Optional key for grouping throttle rules. If not provided, defaults to workflow and subscriber combination."""


class ThrottleControlDto(BaseModel):
    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    type: Optional[ThrottleControlDtoType] = ThrottleControlDtoType.FIXED
    r"""The type of throttle window."""

    amount: Optional[float] = None
    r"""The amount of time for the throttle window (required for fixed type)."""

    unit: Optional[ThrottleControlDtoUnit] = None
    r"""The unit of time for the throttle window (required for fixed type)."""

    dynamic_key: Annotated[Optional[str], pydantic.Field(alias="dynamicKey")] = None
    r"""Key path to retrieve dynamic window value (required for dynamic type)."""

    threshold: Optional[float] = 1
    r"""The maximum number of executions allowed within the window. Defaults to 1."""

    throttle_key: Annotated[Optional[str], pydantic.Field(alias="throttleKey")] = None
    r"""Optional key for grouping throttle rules. If not provided, defaults to workflow and subscriber combination."""
